@@ -6,14 +6,20 @@ from pyroute2 import netlink
 from .preshared_key import PresharedKey
 from .private_key import PrivateKey
 from .public_key import PublicKey
+from .wireguardinfo import WireGuardInfo
 
 class WireGuard(object):
+    __slots__ = ( "__socket" )
+
     from .nlas import device as __device
 
     def __init__(self, **kwargs):
         self.__socket = netlink.generic.GenericNetlinkSocket()
         self.__socket.bind('wireguard', self.__device)
 
+    def __del__(self):
+        self.__socket.close()
+
     def __get(self, device):
         flags = netlink.NLM_F_REQUEST | netlink.NLM_F_DUMP
         return self.__socket.nlm_request(device, msg_type=self.__socket.prid, msg_flags=flags)
@@ -26,59 +32,6 @@ class WireGuard(object):
         device = self.__device.get_device(interface)
         messages = self.__get(device)
 
-        class WireGuardInfo(object):
-            def __init__(self, messages, spill_private_key, spill_preshared_keys):
-                self.ifindex = messages[0].get_attr('WGDEVICE_A_IFINDEX')
-                self.ifname = messages[0].get_attr('WGDEVICE_A_IFNAME')
-                self.private_key = messages[0].get_attr('WGDEVICE_A_PRIVATE_KEY')
-                if not spill_private_key:
-                    self.private_key = self.private_key is not None
-                elif self.private_key is not None:
-                    self.private_key = PrivateKey(self.private_key)
-                self.public_key = messages[0].get_attr('WGDEVICE_A_PUBLIC_KEY')
-                if self.public_key is not None:
-                    self.public_key = PublicKey(self.public_key)
-                self.listen_port = messages[0].get_attr('WGDEVICE_A_LISTEN_PORT')
-                self.fwmark = messages[0].get_attr('WGDEVICE_A_FWMARK')
-
-                self.peers = { }
-
-                for message in messages:
-                    for peer in message.get_attr('WGDEVICE_A_PEERS') or []:
-                        public_key = peer.get_attr('WGPEER_A_PUBLIC_KEY')
-                        assert public_key is not None
-                        if public_key is not None:
-                            public_key = PublicKey(public_key)
-                        if public_key not in self.peers:
-                            preshared_key = peer.get_attr('WGPEER_A_PRESHARED_KEY')
-                            if not spill_preshared_keys:
-                                preshared_key = preshared_key is not None
-                            elif preshared_key is not None:
-                                preshared_key = PresharedKey(preshared_key)
-                            self.peers[public_key] = {
-                                'preshared_key': preshared_key,
-                                'endpoint': peer.get_attr('WGPEER_A_ENDPOINT'),
-                                'persistent_keepalive_interval': peer.get_attr('WGPEER_A_PERSISTENT_KEEPALIVE_INTERVAL'),
-                                'last_handshake_time': peer.get_attr('WGPEER_A_LAST_HANDSHAKE_TIME'),
-                                'rx_bytes': peer.get_attr('WGPEER_A_RX_BYTES'),
-                                'tx_bytes': peer.get_attr('WGPEER_A_TX_BYTES'),
-                                'allowedips': [],
-                                'protocol_version': peer.get_attr('WGPEER_A_PROTOCOL_VERSION'),
-                            }
-                        for allowedip in peer.get_attr('WGPEER_A_ALLOWEDIPS') or []:
-                            self.peers[public_key]['allowedips'].append(allowedip.network())
-
-            def __repr__(self):
-                return repr({
-                    'ifindex': self.ifindex,
-                    'ifname': self.ifname,
-                    'private_key': self.private_key,
-                    'public_key': self.public_key,
-                    'listen_port': self.listen_port,
-                    'fwmark': self.fwmark,
-                    'peers': self.peers,
-                })
-
         return WireGuardInfo(messages, spill_private_key, spill_preshared_keys)
 
     def set_interface(self, interface,
@@ -94,7 +47,9 @@ class WireGuard(object):
             device['attrs'].append(('WGDEVICE_A_FLAGS', device.flag.REPLACE_PEERS.value))
 
         if private_key is not None:
-            if not isinstance(private_key, (bytes, bytearray)):
+            if isinstance(private_key, PrivateKey):
+                private_key = bytes(private_key)
+            elif not isinstance(private_key, (bytes, bytearray)):
                 private_key = bytes(PrivateKey(private_key))
             device['attrs'].append(('WGDEVICE_A_PRIVATE_KEY', private_key))
 
@@ -112,7 +67,9 @@ class WireGuard(object):
 
         for public_key in public_keys:
             peer = self.__device.peer()
-            if not isinstance(public_key, (bytes, bytearray)):
+            if isinstance(public_key, PublicKey):
+                public_key = bytes(public_key)
+            elif not isinstance(public_key, (bytes, bytearray)):
                 public_key = bytes(PublicKey(public_key))
             peer['attrs'].append(('WGPEER_A_PUBLIC_KEY', public_key))
             peer['attrs'].append(('WGPEER_A_FLAGS', peer.flag.REMOVE_ME.value))
@@ -132,7 +89,9 @@ class WireGuard(object):
         device = self.__device.set_device(interface)
 
         peer = device.peer()
-        if not isinstance(public_key, (bytes, bytearray)):
+        if isinstance(public_key, PublicKey):
+            public_key = bytes(public_key)
+        elif not isinstance(public_key, (bytes, bytearray)):
             public_key = bytes(PublicKey(public_key))
         peer['attrs'].append(('WGPEER_A_PUBLIC_KEY', public_key))
 
@@ -148,7 +107,9 @@ class WireGuard(object):
             peer['attrs'].append(('WGPEER_A_FLAGS', flags))
 
         if preshared_key is not None:
-            if not isinstance(preshared_key, (bytes, bytearray)):
+            if isinstance(preshared_key, PresharedKey):
+                preshared_key = bytes(preshared_key)
+            elif not isinstance(preshared_key, (bytes, bytearray)):
                 preshared_key = bytes(PresharedKey(preshared_key))
             peer['attrs'].append(('WGPEER_A_PRESHARED_KEY', preshared_key))
 
@@ -173,7 +134,9 @@ class WireGuard(object):
 
         for public_key in public_keys:
             peer = self.__device.peer()
-            if not isinstance(public_key, (bytes, bytearray)):
+            if isinstance(public_key, PublicKey):
+                public_key = bytes(public_key)
+            elif not isinstance(public_key, (bytes, bytearray)):
                 public_key = bytes(PublicKey(public_key))
             peer['attrs'].append(('WGPEER_A_PUBLIC_KEY', public_key))
             peer['attrs'].append(('WGPEER_A_FLAGS', peer.flag.REPLACE_ALLOWEDIPS.value))
